# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Mathml2latex::MathmlLatexService do
  let(:fixtures_path) { File.join('spec', 'fixtures') }
  let(:service) { Mathml2latex::MathmlLatexService.get_service_object }
  subject { service.to_latex_from_file(mathml_path) }

  describe 'basic to_latex' do
    context 'basic fraction' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'fraction.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('\frac{1}{2}')
      end
    end

    context 'basic square root' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'square_root.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('\sqrt{\mathrm{ss}}')
      end
    end

    context 'basic n root' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'n_root.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('\sqrt[a]{b}')
      end
    end

    context 'basic superscript' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'superscript.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('{a}^{bx}')
      end
    end

    context 'basic left superscript' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'left_superscript.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('{}^{sup}base')
      end
    end

    context 'basic superscript and subscript' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'super_sub_script.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('bas{e}_{sub}^{sup}')
      end
    end

    context 'basic left superscript and subscript' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'left_super_sub_script.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('{}_{sub}{}^{super}base')
      end
    end

    context 'basic subscript' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'subscript.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('bas{e}_{sub}')
      end
    end

    context 'basic left subscript' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'left_subscript.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('{}_{sub}base')
      end
    end

    context 'basic element over' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'element_over.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('\overset{top}{base}')
      end
    end

    context 'basic element under and over' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'element_under_over.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('\underset{under}{\overset{over}{base}}')
      end
    end

    context 'basic element under' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'element_under.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('\underset{under}{base}')
      end
    end

    context 'basic under script with brace' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'under_script_with_brace.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('\underset{under}{\underbrace{up}}')
      end
    end

    context 'basic over script with brace' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'over_script_with_brace.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('\overset{top}{\overbrace{base}}')
      end
    end

    context 'basic superscript and subscript with big operator' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'super_sub_script_big_operator.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('{\mbox{\large$base$}}_{under}^{over}')
      end
    end

    context 'basic subscript with big operator' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'subscript_with_big_operator.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('{\mbox{\large$base$}}_{sub}')
      end
    end

    context 'basic backspace' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'backspace.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('223444\phantom{\rule{-0.2em}{0ex}}333\phantom{\rule{-0.2em}{0ex}}')
      end
    end

    context 'basic digit space' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'digit_space.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq('123\, 456\phantom{\rule{-0.2em}{0ex}}')
      end
    end

    context 'basic greek characters' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'greek.xml') }

      it 'outputs correct latex' do
        expect(subject).to match(
          /\\alpha \\beta \\gamma \\delta \\varepsilon \\zeta \\eta/
        )
        expect(subject).to match(
          /\\theta \\vartheta \\iota \\kappa \\lambda \\mu \\nu \\xi o \\pi/
        )
        expect(subject).to match(
          /\\varpi \\rho \\varsigma \\sigma \\tau \\upsilon \\varphi \\phi \\chi \\psi \\omega/
        )
      end
    end

    context 'basic matrix' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'matrix.xml') }

      it 'outputs correct latex' do
        # actual result: \begin{array}{ccc}11& 12& 13\\ 21& 22& 23\end{array}
        expect(subject).to eq('\begin{array}{ccc}11& 12& 13\\\\ 21& 22& 23\end{array}')
      end
    end

    context 'basic matrix with vertical bar' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'matrix_with_vertical_bar.xml') }

      it 'outputs correct latex' do
        # actual result: \left|\begin{array}{cc}11& 22\\ 33& 44\\ 55& 66\end{array}\right|
        expect(subject).to eq('\left|\begin{array}{cc}11& 22\\\\ 33& 44\\\\ 55& 66\end{array}\right|')
      end
    end

    context 'basic matrix with square brackets' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'matrix_square_brackets.xml') }

      it 'outputs correct latex' do
        # actual result: \left[\begin{array}{cc}11& 12\\ 21& 22\\ 31& 32\end{array}\right]
        expect(subject).to eq('\left[\begin{array}{cc}11& 12\\\\ 21& 22\\\\ 31& 32\end{array}\right]')
      end
    end

    context 'basic matrix with parenthesis' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'matrix_parenthesis.xml') }

      it 'outputs correct latex' do
        # actual result: \left(\begin{array}{ccc}11& 12& 13\\ 21& 22& 23\end{array}\right)
        expect(subject).to eq('\left(\begin{array}{ccc}11& 12& 13\\\\ 21& 22& 23\end{array}\right)')
      end
    end

    context 'basic piecewise function' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'piecewise_function.xml') }

      it 'outputs correct latex' do
        # actual result: \left\{\begin{array}{ll}11& 22\\ 33& 44\end{array}\right.
        expect(subject).to eq('\left\{\begin{array}{ll}11& 22\\\\ 33& 44\end{array}\right.')
      end
    end

    context 'basic aligned function' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'aligned_equations.xml') }

      it 'outputs correct latex' do
        # actual result: \begin{array}{rcl}x\operatorname ~ +~ 2y& =& 25\\ 3x\operatorname ~ +\, 6y& =& 20\end{array}
        expect(subject).to eq(
          '\begin{array}{rcl}x\operatorname ~ +~ 2y& =& 25\\\\ 3x\operatorname ~ +\, 6y& =& 20\end{array}'
        )
      end
    end

    context 'basic parenthesis' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'parenthesis.xml') }

      it 'outputs correct latex' do
        # actual result: \left(www\right)
        expect(subject).to eq('\left(www\right)')
      end
    end

    context 'basic double vertical bars' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'double_vertical_bars.xml') }

      it 'outputs correct latex' do
        # actual result: \left\|eeee\right\|
        expect(subject).to eq('\left\|eeee\right\|')
      end
    end

    context 'basic matrix with curly brakets' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'matrix_curly_brackets.xml') }

      it 'outputs correct latex' do
        # actual result: \left\{\begin{array}{cc}11& 22\\ 33& 44\\ 55& 66\end{array}\right\}
        expect(subject).to eq('\left\{\begin{array}{cc}11& 22\\\\ 33& 44\\\\ 55& 66\end{array}\right\}')
      end
    end

    context 'basic matrix with top curly braket' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'matrix_top_curly_bracket.xml') }

      it 'outputs correct latex' do
        # actual result: \overbrace{\begin{array}{ccc}11& 22& 33\\ aa& bb& cc\end{array}}
        expect(subject).to eq('\overbrace{\begin{array}{ccc}11& 22& 33\\\\ aa& bb& cc\end{array}}')
      end
    end

    context 'basic matrix with bottom curly braket' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'matrix_bottom_curly_bracket.xml') }

      it 'outputs correct latex' do
        # actual result: \underbrace{\begin{array}{ccc}11& 22& 33\\ aa& bb& cc\end{array}}
        expect(subject).to eq('\underbrace{\begin{array}{ccc}11& 22& 33\\\\ aa& bb& cc\end{array}}')
      end
    end

    context 'basic top parenthesis' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'top_parenthesis.xml') }

      it 'outputs correct latex' do
        # actual result: \overset{\frown }{2233}
        expect(subject).to eq('\overset{\frown }{2233}')
      end
    end

    context 'basic bottom parenthesis' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'bottom_parenthesis.xml') }

      it 'outputs correct latex' do
        # actual result: \underset{\smile }{abcde}
        expect(subject).to eq('\underset{\smile }{abcde}')
      end
    end

    context 'basic vector accent' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'vector_accent.xml') }

      it 'outputs correct latex' do
        # actual result: \overset{\rightharpoonup }{vector}
        expect(subject).to eq('\overset{\rightharpoonup }{vector}')
      end
    end

    context 'basic arrow accent' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'arrow_accent.xml') }

      it 'outputs correct latex' do
        # actual result: \overset{\rightarrow }{abcd}
        expect(subject).to eq('\overset{\rightarrow }{abcd}')
      end
    end

    context 'basic left right arrow accent' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'left_right_arrow_accent.xml') }

      it 'outputs correct latex' do
        # actual result: \overset{\leftrightarrow }{abcde}
        expect(subject).to eq('\overset{\leftrightarrow }{abcde}')
      end
    end

    context 'basic enclose left' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'enclose_left.xml') }

      it 'outputs correct latex' do
        # actual result: \left|abcde\right.
        expect(subject).to eq('\left|abcde\right.')
      end
    end

    context 'basic enclose right' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'enclose_right.xml') }

      it 'outputs correct latex' do
        # actual result: \left.{eeee}\right|
        expect(subject).to eq('\left.{eeee}\right|')
      end
    end

    context 'basic enclose box matrix' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'enclose_box_matrix.xml') }

      it 'outputs correct latex' do
        # actual result: \boxed{\begin{array}{ccc}11& 22& 33\\ 44& 55& 66\end{array}}
        expect(subject).to eq('\boxed{\begin{array}{ccc}11& 22& 33\\\\ 44& 55& 66\end{array}}')
      end
    end

    context 'basic sum with superscript and subscript' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'sum_super_sub.xml') }

      it 'outputs correct latex' do
        # actual result: {\textstyle\sum _{67}^{12}}
        expect(subject).to eq('{\textstyle\sum _{67}^{12}}')
      end
    end

    context 'basic sum with over and under' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'sum_over_under.xml') }

      it 'outputs correct latex' do
        # actual result: \sum _{aa}^{bb}
        expect(subject).to eq('\sum _{aa}^{bb}')
      end
    end

    context 'basic definite integral with differential' do
      let(:mathml_path) { File.join(fixtures_path, 'basic', 'definite_integral_with_differential.xml') }

      it 'outputs correct latex' do
        # actual result: {\int }_{aa}^{bb}cc\operatorname dx
        expect(subject).to eq('{\int }_{aa}^{bb}cc\operatorname dx')
      end
    end
  end

  describe 'complex latex' do
    context 'sum with nested matrixes' do
      let(:mathml_path) { File.join(fixtures_path, 'complex', 'sum_with_nested_matrix.xml') }

      it 'outputs correct latex' do
        expect(subject).to eq(
          '{\textstyle\sum _{\left|\begin{array}{ccc}1& 2& 3\\\\ 4& \left\{\begin{array}'\
          '{l}11\\\\ \left[\begin{array}{ccc}1& 2& 3\\\\ 4& 5& {\int }_{a}^{b}x\operatorname'\
          ' dx\end{array}\right]\end{array}\right.& 6\end{array}\right|}}'
        )
      end
    end
  end
end
